import json
import requests

import quart

from .errors import *

class Dashboard:	
	def __init__(self, bot, key, url, *args, **kwargs):
		"""Initialize the dashboard"""
		self.bot = bot
		self.key = key
		self.url = url
		self.routes = {}
		
		
	def route(self, func):
		if func.__name__ in self.routes.keys():
			raise DuplicateRoute("that route already exists")
		self.routes[func.__name__] = func
		
		
	async def process_request(self, message):
		try:
			r = json.loads(message.content)
		
		except:
			return
			
		if r.get("Authorization") != self.key:
			return
			
		
		if r.get("route") not in self.routes.keys():
			raise RouteNotFound("the requested route does not exist")
			
		
		else:
			result = await self.routes[r["route"]](r.get("data"))
			
			requests.post(self.url, headers={"Authorization": self.key}, json={"name": r["route"], "data": result})
			
		return {"error": False, "message": "Request processed successfully"}




import disnake
from disnake import Webhook, AsyncWebhookAdapter
import aiohttp
import json
import asyncio

from .errors import *

class Server:
	def __init__(self, server, key, *args, **kwargs):
		"""Initialize the server"""
		if not kwargs.get("webhook_url"):
			raise TypeError("__init__() missing one required positional argument: 'webhook_url'")
		
		self.server = server	
		self.key = key
		self.webhook_url = kwargs.get("webhook_url")
		self.sleep_time = kwargs.get("sleep_time") or 1
		self.data = {}
		
		
	async def request(self, route, *args, **kwargs):
		"""Send a request to the bot"""
		request_data = {"Authorization": self.key, "route": route, "data": kwargs}
		
		request = json.dumps(request_data)
		
		async with aiohttp.ClientSession() as session:
			webhook = Webhook.from_url(self.webhook_url, adapter=AsyncWebhookAdapter(session))
			await webhook.send(request)
			
		await asyncio.sleep(self.sleep_time)
		
		data = self.data.pop(route, None)

		return data

	
	async def process_request(self, request):
		"""Processes the returned data from the bot"""
		r = await request.json
		
		if request.headers.get("Authorization") != self.key:
			raise InvalidAuthorization("invalid key provided")
			
		if r.get("name"):
			self.data[r["name"]] = r["data"]



from setuptools import setup
import re
import os

version = ""
with open("disnake/ext/dashboard/__init__.py") as f:
	version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

if not version:
	raise RuntimeError("version is not set")
		

readme = '''
## Установка
```py
pip install --upgrade disnake-ext-dashboard

python3 -m pip install --upgrade disnake-ext-dashboard
```

## Применение
### Предпосылки
Прежде чем приступить к работе, вам понадобится несколько вещей:
 - Вебхук в секретном канале (если у кого-то есть доступ, он сможет все получить лягушку а это плохо).
 - Правильно размещенный бот [**disnake.py**](https://github.com/DisnakeDev/disnake)
 
 И так поехали!(жабы топ)

### Примеры
#### Бот
```py
import disnake
from disnake.ext import commands
from disnake.ext.dashboard import Dashboard

bot = commands.Bot(command_prefix="!")
bot_dashboard = Dashboard(bot,
	"secret_key", 
	"https://your-bot-website.com/dashboard"
)

@bot.event
async def on_ready():
	print(f"Bot is online as {bot.user}")

@bot.event
async def on_message(message):
	await bot_dashboard.process_request(message)

@bot_dashboard.route
async def guild_count(data):
	return len(bot.guilds)

@bot_dashboard.route
async def member_count(data):
	return await bot.fetch_guild(data["guild_id"]).member_count

bot.run("your-token-here")
```


#### Веб-сервер
```py
from quart import Quart, request
from disnake.ext.dashboard import Server

app = Quart(__name__)
app_dashboard = Server(
	app,
	"secret_key", 
	webhook_url="https://your-private-discord-webhook.com",
	sleep_time=1
)

@app.route("/")
async def index():
	guild_count = await app_dashboard.request("guild_count")
	member_count = await app_dashboard.request("member_count", guild_id=776230580941619251)

	return f"Guild count: {guild_count}, Server member count: {member_count}"

@app.route("/dashboard", methods=["POST"])
async def dashboard():
	# Don't worry about authorization, the bot will handle it
	await app_dashboard.process_request(request)
        
        
if __name__ == "__main__":
        app.run()
```


Обратите внимание, что Cogs в настоящее время не поддерживаются.'''

	

requirements = ["disnake>=2.7.0"]

setup(name='disnake-ext-dashboard',
      author='MaseZev',
      url='https://github.com/MaseZev/disnake-ext-dashboard',
      version=version,
      packages=['disnake.ext.dashboard'],
      license='MIT',
      description='Веб-перехватчик и расширение disnake на основе запросов для создания панели управления ботом.',
      long_description=readme,
      long_description_content_type="text/markdown",
      install_requires=requirements,
      python_requires='>=3.8.0',
      classifiers=[
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
      ]
)
